% Code for ultrasound tomography simulation, metallic cylinder example
% Author: Stefano Franceschini
% date of the current version: 28/04/2022

clc
clear all
close all

%=================================
% Computational Grid
%=================================

addpath('C:\Users\ToniaLabTLC\Desktop\Dottorato_di_ricerca\Us_2D\k-Wave'); %path of the K-Wave folder
Nx = 472;           % number of grid points in the x (row) direction
Ny = Nx;           % number of grid points in the y (column) direction
dx = 1.5e-3;        % grid point spacing in the x direction [m]
dy=dx;              % grid point spacing in the y direction [m]
kgrid = kWaveGrid(Nx, dx, Ny, dy);
kgrid.Nt = 1500000; % number of samples of the signal
f0=4e4;             % frequency of the source
kgrid.dt=1/1e9;     % time step

%=================================
% Medium acoustic properties
%=================================

medium.density=1.225+zeros(Nx,Ny); %air density
medium.sound_speed=343+zeros(Nx,Ny); % air speed of sound
medium.alpha_coeff=0.0004+zeros(Nx,Ny); % air attenuation coefficient
medium.alpha_power=1.001;

radius1=0.002; %radius of the iron cylinder
cx=242; cy=199; %position of the cylinder
iron_cyl=makeDisc(Nx, Ny, cx,cy,radius1/dx);
medium.density(iron_cyl==1)=7880; %iron density
medium.sound_speed(iron_cyl==1)=3200; % iron speed of sound
medium.alpha_coeff(iron_cyl==1)=2; %iron attenuation coefficient

%=================================
% Sensors
%=================================

sensor.mask=zeros(Nx,Ny);
th = 0:pi/11:2*pi-pi/154;    % angular positions of the sensors
sens_rad=0.168/dx;
yunit = round(sens_rad.* sin(th))+Nx/2;
xunit = round(sens_rad.* cos(th))+Nx/2; 

for j=1:length(xunit)
    
    sensor.mask(xunit(j),yunit(j))=1;

end

% remove the transmitters from the receiver list

tx1_x=220; 
tx1_y=347;
tx2_x=129;
tx2_y=204;
tx3_x=252;
tx3_y=125;
tx4_x=343;
tx4_y=268;

sensor.mask(tx1_x,tx1_y)=0;
sensor.mask(tx2_x,tx2_y)=0;
sensor.mask(tx3_x,tx3_y)=0;
sensor.mask(tx4_x,tx4_y)=0;

% store the positions of the transmitters

arc_pos(:,1) = [tx1_x,tx1_y];          
arc_pos(:,2) = [tx2_x,tx2_y];           
arc_pos(:,3) = [tx3_x,tx3_y];         
arc_pos(:,4) = [tx4_x,tx4_y];         

% set the pattern of the transmitters

focus_pos = [Nx/2, Ny/2];   % focus point of the transmitter (the center of the ROI) [grid points]
radius =50;                % [grid points] 
diameter =45;              % [grid points]

% set the pattern of the receivers

[aa,bb]=find(sensor.mask==1);
x_angle=bb-(Nx/2);
y_angle=aa-(Ny/2);
dir_angles=atan2(x_angle,y_angle);
sensor.directivity_angle = zeros(Nx, Ny);
sensor.directivity_angle(sensor.mask == 1) = dir_angles';
sensor.directivity_pattern = 'pressure';
sensor.directivity_size = 6* kgrid.dx; 

%=================================
% Source
%=================================

source_mag = 20;         % [Pa]
source.p = source_mag * sin(2 * pi * f0* kgrid.t_array);
temp=source.p; %store the source
temp(1:1000000)=[]; %delete the transient
tx_anal=hilbert(temp); %compute analytic signal


%===================================================
% Run Simulation
%===================================================

input_args = {};
nt=4;  %number of transmitters
nr=18; %number of receivers

for ii=1:nt
    
    source.p_mask = makeArc([Nx, Ny], arc_pos(:,ii)', radius, diameter, focus_pos);
    sensor_data=kspaceFirstOrder2D(kgrid, medium, source, sensor,'PlotLayout', false,'PlotPML', true,'PMLAlpha',2,'PMLSize',20,'PMLInside',false,'PlotSim',false);  
    
    for k=1:nr
        
        tt=sensor_data(k,:);
        tt(1:1000000)=[]; %delete the transient
        rx_anal=hilbert(tt); %compute analytic signal
        batt(k,ii)=mean(tx_anal.*conj(rx_anal)); %mean beat extraction
    
    end
    
end
[batt2, indices_new] = reorderSensorData(kgrid, sensor, batt);
